"""\
Various useful plugins for working with bzr.
"""
import bzrlib.commands
import push
import annotate
import shelf
import sys
import os.path
sys.path.append(os.path.dirname(__file__))

bzrlib.commands.OPTIONS['ignored'] = None

class cmd_clean_tree(bzrlib.commands.Command):
    """Remove unwanted files from working tree.
    Normally, ignored files are left alone.  The --ignored flag will cause them
    to be deleted as well.
    """
    takes_options = ['ignored']
    def run(self, ignored=False):
        import clean_tree
        clean_tree.clean_tree(ignored=ignored)

bzrlib.commands.OPTIONS['no-collapse'] = None
bzrlib.commands.OPTIONS['no-antialias'] = None
bzrlib.commands.OPTIONS['cluster'] = None
bzrlib.commands.OPTIONS['merge-branch'] = str

class cmd_graph_ancestry(bzrlib.commands.Command):
    """Produce ancestry graphs using dot.
    
    Output format is detected according to file extension.  Some of the more
    common output formats are png, gif, svg, ps.  An extension of '.dot' will
    cause a dot graph file to be produced.

    Branches are labeled r?, where ? is the revno.  If they have no revno,
    with the last 5 characters of their revision identifier are used instead.
    
    If --merge-branch is specified, the two branches are compared and a merge
    base is selected.
    
    Legend:
    white    normal revision
    yellow   THIS  history
    red      OTHER history
    orange   COMMON history
    blue     COMMON non-history ancestor
    dotted   Missing from branch storage

    Ancestry is usually collapsed by removing revisions with a single parent
    and descendant.  The number of skipped revisions is shown on the arrow.
    This feature can be disabled with --no-collapse.

    By default, revisions are ordered by distance from root, but they can be
    clustered instead using --cluster.

    If available, rsvg is used to antialias PNG and JPEG output, but this can
    be disabled with --no-antialias.
    """
    takes_args = ['branch', 'file']
    takes_options = ['no-collapse', 'no-antialias', 'merge-branch', 'cluster']
    def run(self, branch, file, no_collapse=False, no_antialias=False,
        merge_branch=None, cluster=False):
        import graph
        if cluster:
            ranking = "cluster"
        else:
            ranking = "forced"
        graph.write_ancestry_file(branch, file, not no_collapse, 
                                  not no_antialias, merge_branch, ranking)

class cmd_fetch_ghosts(bzrlib.commands.Command):
    """Attempt to retrieve ghosts from another branch.
    If the other branch is not supplied, the last-pulled branch is used.
    """
    aliases = ['fetch-missing']
    takes_args = ['branch?']
    def run(self, branch=None):
        from fetch_ghosts import fetch_ghosts
        fetch_ghosts(branch)

class cmd_patch(bzrlib.commands.Command):
    """Apply a named patch to the current tree.
    """
    takes_args = ['filename?']
    takes_options = ['strip']
    def run(self, filename=None, strip=0):
        from patch import patch
        from bzrlib.branch import Branch
        b = Branch.open_containing('.')
        return patch(b, filename, strip)



commands = [push.cmd_push, annotate.cmd_annotate, shelf.cmd_shelve, 
            shelf.cmd_unshelve, cmd_clean_tree, cmd_graph_ancestry,
            cmd_fetch_ghosts, cmd_patch]
from errors import NoPyBaz
try:
    import baz_import
    commands.append(baz_import.cmd_baz_import)

except NoPyBaz:
    class cmd_baz_import(bzrlib.commands.Command):
        """Disabled. (Requires PyBaz)"""
    commands.append(cmd_baz_import)

if hasattr(bzrlib.commands, 'register_command'):
    for command in commands:
        bzrlib.commands.register_command(command)

def test_suite():
    from doctest import DocTestSuite
    return DocTestSuite(bzrtools)
